local L = LibStub("AceLocale-3.0"):GetLocale("OneCore3")
local AceDB3 = LibStub('AceDB-3.0')
local AceConfig3 = LibStub('AceConfig-3.0')
local AceCmd3 = LibStub('AceConfigCmd-3.0')

local artDescriptions = {}
local artCount = 3

artDescriptions[1] = L["Normal"]
artDescriptions[2] = L["Silver Dragon"]
artDescriptions[3] = L["Gold Dragon"]

PlayerFramePlus = LibStub("AceAddon-3.0"):NewAddon("KC_PlayerFramePlus", "AceEvent-3.0", "AceHook-3.0", "AceConsole-3.0")

function PlayerFramePlus:OnInitialize(name)
	self.db = AceDB3:New("KC_PlayerFramePlusDB")
	self.db:RegisterDefaults({
		profile = {
			showText = true, 
			artid = 1
		}
	})
	
	local args	   = {
		type = "group", 
		args = {
			showtext = { 
				name = L["Show Text"], 
				desc = L["Toggles the display of experiance text."], 
				get = function(info)
					return self.db.profile.showText
				end, 
				set = function(info, shown)
					self.db.profile.showText = shown
					if not shown then
						self.frame.xpText:SetText("")
					else
						self:ShowExp()
					end
					self:Print(L["PlayerFrame experiance text has been toggled."])
				end, 
				type = "toggle",  
			},
			art	= {
				name = L["Cycle Art"],
				desc = L["Cycles the unit frame artwork"],
				type = "execute",
				func = function()
					local id = self.db.profile.artid + 1
					if id > artCount then id = 1 end
					self:SetArt(id)
					self.db.profile.artid = id
					self:Print(L["PlayerFrame artwork is now set to [%s]"]:format(artDescriptions[id]))
				end
			}
		}
	}
	
	AceConfig3:RegisterOptionsTable("KC_PlayerFramePlus", args, 'kcpf')
	AceCmd3:CreateChatCommand("kcpfp", "KC_PlayerFramePlus")
	AceCmd3:CreateChatCommand("kc_playerframeplus", "KC_PlayerFramePlus")
	self:SetArt()
end


function PlayerFramePlus:OnEnable()
	if UnitLevel("player") == 80 then
		return
	end
	
	self:CreateFrame()	
	self:ShowExp()
	
	-- Makes TargetFrame Dragable
	self:HookScript(TargetFrame, "OnMouseDown", "StartMoving")
	self:HookScript(TargetFrame, "OnMouseUp", "StopMoving")

	-- Makes PetFrame Dragable	  
	self:HookScript(PetFrame, "OnMouseDown", "StartMoving")
	self:HookScript(PetFrame, "OnMouseUp", "StopMoving")

	self:RegisterEvent("PLAYER_XP_UPDATE", "ShowExp")
	self:RegisterEvent("PLAYER_LEVEL_UP", "ShowExp")
	self:RegisterEvent("UPDATE_EXHAUSTION", "ShowExp")
end

function PlayerFramePlus:CreateFrame()

	frame = CreateFrame('Frame', nil, PlayerFrame)
	
	border = frame:CreateTexture(nil, "OVERLAY")
	
	border:SetHeight(128)
	border:SetWidth(256)
	border:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 77, -37)
	border:SetTexture[[Interface\AddOns\KC_PlayerFramePlus\Art\KC_PlayerFramePlus]]
	
	xpText = frame:CreateFontString(nil, "OVERLAY",  "GameFontNormalSmall")
	xpText:SetPoint("CENTER", border, "CENTER", 0, 30)
	xpText:SetTextColor(.9, .7, 1)
	
	frame.xpText = xpText
	
	xpBar = CreateFrame("StatusBar", nil, frame, "TextStatusBar")
	xpBar:SetWidth(218)
	xpBar:SetHeight(12)
	xpBar:SetPoint("TOPLEFT", border, "TOPLEFT", 12, -28)
	xpBar:SetStatusBarTexture[[Interface\TargetingFrame\UI-StatusBar]]
	xpBar:SetStatusBarColor(.5, 0, .75, 1)
	xpBar:SetFrameLevel("1")
	
	frame.xpBar = xpBar
	
	bg = xpBar:CreateTexture(nil, "BACKGROUND")
	bg:SetTexture(0, 0, 0, .5)
	bg:SetWidth(218)
	bg:SetHeight(12)
	bg:SetPoint("BOTTOM", PlayerFrame, "BOTTOM", 84, 22)

	self.frame = frame
end

function PlayerFramePlus:ShowExp()
	local currXP, nextXP, restXP, str = UnitXP("player"), UnitXPMax("player"), GetXPExhaustion()
	
	if self.db.profile.showText then
		if(restXP == nil) then
			str = format("%s / %sxp %s%%", currXP, nextXP, floor((tonumber(currXP) / tonumber(nextXP)) * 100))
		else
			str = format("%s / %sxp %s%% (+%sxp) ", currXP, nextXP, floor((tonumber(currXP) / tonumber(nextXP)) * 100),(tonumber(restXP) / 2))
		end		
		
		self.frame.xpText:SetText(str)
	end
		
	self.frame.xpBar:SetMinMaxValues(min(0, currXP), nextXP)
	self.frame.xpBar:SetValue(currXP)
end


function PlayerFramePlus:StartMoving(target)
	if (IsAltKeyDown()) then target:SetMovable(true); target:StartMoving(); target:IsUserPlaced(true) end
end


function PlayerFramePlus:StopMoving(target)
	target:StopMovingOrSizing()
end

function PlayerFramePlus:SetArt(id)
	if not id then id = self.db.profile.artid end
	PlayerFrameTexture:SetTexture("Interface\\AddOns\\KC_PlayerFramePlus\\Art\\"..id)
end